package com.uas.search.dao;

import com.uas.search.model.Component;
import com.uas.search.model.Goods;
import com.uas.search.model.Store;
import com.uas.search.model.TradeGoods;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Repository;
import org.springframework.util.CollectionUtils;

import java.util.ArrayList;
import java.util.List;

/**
 * @author sunyj
 * @since 2017年7月8日 下午7:45:01
 */
@Repository
public class GoodsDao {

	@Autowired
	private TradeGoodsDao tradeGoodsDao;

	@Autowired
	private ComponentDao componentDao;

	@Autowired
	private StoreDao storeDao;

	/**
	 * 根据批次id或者器件id获取批次
	 *
	 * @param goods
	 * @return
	 */
	public List<Goods> find(Goods goods) {
		if (goods == null) {
			return null;
		}
		List<Goods> goodsesList = new ArrayList<>();
		if (goods.getGoId() != null) {
			goodsesList.add(findByGoId(goods.getGoId()));
		} else if (goods.getCmpId() != null) {
			goodsesList.addAll(findByCmpId(goods.getCmpId()));
		}
		return goodsesList;
	}

	/**
	 * 根据批次id获取批次信息
	 *
	 * @param goId
	 * @return
	 */
	private Goods findByGoId(Long goId) {
		TradeGoods tradeGoods = tradeGoodsDao.findOne(goId);
		Component component = componentDao.findByUuid(tradeGoods.getCmpUuid());
		Store store = null;
		if (tradeGoods.getStoreId() != null) {
			store = storeDao.findByUuid(tradeGoods.getStoreId());
		}
		return new Goods(tradeGoods, store, component);
	}

	/**
	 * 根据器件id获取批次信息
	 *
	 * @param cmpId
	 * @return
	 */
	public List<Goods> findByCmpId(Long cmpId) {
		Component component = componentDao.findOne(cmpId);
		if(component == null){
			throw new IllegalStateException("器件不存在：" + cmpId);
		}
		return findByComponent(component);
	}

	/**
	 * 根据器件获取批次信息
	 *
	 * @param component
	 * @return
	 */
	public List<Goods> findByComponent(Component component) {
		if(component == null){
			return null;
		}
		List<TradeGoods> tradeGoodsesList = tradeGoodsDao.findByCmpUuid(component.getUuid());
		List<Goods> goodsesList = new ArrayList<>();
		if (!CollectionUtils.isEmpty(tradeGoodsesList)) {
			for (TradeGoods tradeGoods : tradeGoodsesList) {
				Store store = null;
				if (tradeGoods.getStoreId() != null) {
					store = storeDao.findByUuid(tradeGoods.getStoreId());
				}
				goodsesList.add(new Goods(tradeGoods, store, component));
			}
		} else {
			goodsesList.add(new Goods(null, null, component));
		}
		return goodsesList;
	}
}
